@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.65 2003/09/05 21:22:24 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.76 2003/10/10 20:43:48 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -27,6 +27,7 @@
 #include <math.h>
 #include <string.h>
 #include <stdio.h>
+#include <stdlib.h>
 /* X11 */
 #include <X11/X.h>
 #include <X11/Xlib.h>
@@ -65,6 +66,7 @@ enum {
 	TICK,
 	GOT_METADATA,
 	BUFFERING,
+	SPEED_WARNING,
 	LAST_SIGNAL,
 };
 
@@ -76,6 +78,7 @@ enum {
 	CHANNELS_CHANGE_ASYNC,
 	BUFFERING_ASYNC,
 	MESSAGE_ASYNC,
+	SPEED_WARNING_ASYNC,
 };
 
 typedef struct {
@@ -99,11 +102,6 @@ enum {
 	PROP_SHOW_VISUALS,
 };
 
-static int speeds[2] = {
-	XINE_SPEED_PAUSE,
-	XINE_SPEED_NORMAL,
-};
-
 static int video_props[2] = {
 	XINE_PARAM_VO_BRIGHTNESS,
 	XINE_PARAM_VO_CONTRAST,
@@ -141,6 +139,11 @@ struct BaconVideoWidgetPrivate {
 	GList *visuals;
 	char *queued_vis;
 
+	/* Seeking stuff */
+	int seeking;
+	float seek_dest;
+	gint64 seek_dest_time;
+
 	/* Other stuff */
 	int xpos, ypos;
 	gboolean can_dvd, can_vcd, can_cdda;
@@ -150,6 +153,7 @@ struct BaconVideoWidgetPrivate {
 	int volume;
 	TvOutType tvout;
 	guint32 video_fcc, audio_fcc;
+	gboolean is_live;
 
 	GAsyncQueue *queue;
 	int video_width, video_height;
@@ -206,6 +210,8 @@ static GtkWidgetClass *parent_class = NULL;
 
 static void xine_event (void *user_data, const xine_event_t *event);
 static gboolean bacon_video_widget_idle_signal (BaconVideoWidget *bvw);
+static void show_vfx_update (BaconVideoWidget *bvw, gboolean show_visuals);
+
 
 static int bvw_table_signals[LAST_SIGNAL] = { 0 };
 
@@ -266,16 +272,12 @@ bacon_video_widget_class_init (BaconVideoWidgetClass *klass)
 	g_object_class_install_property (object_class, PROP_LOGO_MODE,
 			g_param_spec_boolean ("logo_mode", NULL, NULL,
 				FALSE, G_PARAM_READWRITE));
-	g_object_class_install_property (object_class, PROP_SPEED,
-			g_param_spec_int ("speed", NULL, NULL,
-				SPEED_PAUSE, SPEED_NORMAL,
-				0, G_PARAM_READWRITE));
 	g_object_class_install_property (object_class, PROP_POSITION,
-			g_param_spec_int ("position", NULL, NULL,
-				0, 65535, 0, G_PARAM_READABLE));
+			g_param_spec_int64 ("position", NULL, NULL,
+				0, G_MAXINT64, 0, G_PARAM_READABLE));
 	g_object_class_install_property (object_class, PROP_STREAM_LENGTH,
-			g_param_spec_int ("stream_length", NULL, NULL,
-				0, 65535, 0, G_PARAM_READABLE));
+			g_param_spec_int64 ("stream_length", NULL, NULL,
+				0, G_MAXINT64, 0, G_PARAM_READABLE));
 	g_object_class_install_property (object_class, PROP_PLAYING,
 			g_param_spec_boolean ("playing", NULL, NULL,
 				FALSE, G_PARAM_READABLE));
@@ -342,9 +344,9 @@ bacon_video_widget_class_init (BaconVideoWidgetClass *klass)
 				G_SIGNAL_RUN_LAST,
 				G_STRUCT_OFFSET (BaconVideoWidgetClass, tick),
 				NULL, NULL,
-				baconvideowidget_marshal_VOID__INT_INT_INT,
-				G_TYPE_NONE, 3, G_TYPE_INT, G_TYPE_INT,
-				G_TYPE_INT);
+				baconvideowidget_marshal_VOID__INT64_INT64_FLOAT,
+				G_TYPE_NONE, 3, G_TYPE_INT64, G_TYPE_INT64,
+				G_TYPE_FLOAT);
 
 	bvw_table_signals[BUFFERING] =
 		g_signal_new ("buffering",
@@ -354,6 +356,15 @@ bacon_video_widget_class_init (BaconVideoWidgetClass *klass)
 				NULL, NULL,
 				g_cclosure_marshal_VOID__INT,
 				G_TYPE_NONE, 1, G_TYPE_INT);
+
+	bvw_table_signals[SPEED_WARNING] =
+		g_signal_new ("speed-warning",
+				G_TYPE_FROM_CLASS (object_class),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (BaconVideoWidgetClass, speed_warning),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__VOID,
+				G_TYPE_NONE, 0);
 }
 
 static void
@@ -379,12 +390,12 @@ bacon_video_widget_instance_init (BaconVideoWidget *bvw)
 	bvw->priv->queue = g_async_queue_new ();
 
 	/* init configuration  */
+	bvw->priv->gc = gconf_client_get_default ();
 	setup_config (bvw);
 
 	xine_init (bvw->priv->xine);
 
 	/* Debug configuration */
-	bvw->priv->gc = gconf_client_get_default ();
 	if (gconf_client_get_bool (bvw->priv->gc, GCONF_PREFIX"/debug", NULL) == FALSE)
 	{
 		xine_engine_set_param (bvw->priv->xine,
@@ -625,6 +636,35 @@ load_audio_out_driver (BaconVideoWidget *bvw, GError **err)
 	return ao_driver;
 }
 
+static void
+bvw_config_helper_string (xine_t *xine, const char *id, const char *val,
+		xine_cfg_entry_t *entry)
+{
+	memset (entry, 0, sizeof (entry));
+
+	if (!xine_config_lookup_entry (xine, id, entry))
+	{
+		xine_config_register_string (xine, id, val, "", NULL, 10,
+				NULL, NULL);
+		xine_config_lookup_entry (xine, id, entry);
+	}
+}
+
+static void
+bvw_config_help_num (xine_t *xine, const char *id, int val,
+		xine_cfg_entry_t *entry)
+{
+	memset (entry, 0, sizeof (entry));
+
+	if (!xine_config_lookup_entry (xine, id, entry))
+	{
+		xine_config_register_num (xine, id, val, 0, NULL, 10,
+				NULL, NULL);
+		xine_config_lookup_entry (xine, id, entry);
+	}
+}
+
+
 static void
 setup_config (BaconVideoWidget *bvw)
 {
@@ -662,76 +702,57 @@ setup_config (BaconVideoWidget *bvw)
 	if (bvw->priv->gc == NULL)
 		return;
 
-	memset (&entry, 0, sizeof (entry));
-
 	if (gconf_client_get_bool (bvw->priv->gc, "/system/http_proxy/use_http_proxy", NULL) == FALSE)
 	{
-		if (xine_config_lookup_entry (bvw->priv->xine,
-				"input.http_proxy_host", &entry))
-		{
-			entry.str_value = "";
-			xine_config_update_entry (bvw->priv->xine, &entry);
-		}
+		bvw_config_helper_string (bvw->priv->xine,
+				"input.http_proxy_host", "", &entry);
+		entry.str_value = "";
+		xine_config_update_entry (bvw->priv->xine, &entry);
 
 		return;
 	}
 
-	memset (&entry, 0, sizeof (entry));
-	if (xine_config_lookup_entry (bvw->priv->xine,
-			"input.http_proxy_host", &entry))
-	{
-		entry.str_value = gconf_client_get_string (bvw->priv->gc,
-				"/system/http_proxy/host", NULL);
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_helper_string (bvw->priv->xine,
+			"input.http_proxy_host", "", &entry);
+	entry.str_value = gconf_client_get_string (bvw->priv->gc,
+			"/system/http_proxy/host", NULL);
+	xine_config_update_entry (bvw->priv->xine, &entry);
 
-	memset (&entry, 0, sizeof (entry));
-	if (xine_config_lookup_entry (bvw->priv->xine,
-			"input.http_proxy_port", &entry))
-	{
-		entry.num_value = gconf_client_get_int (bvw->priv->gc,
-				"/system/http_proxy/port", NULL);
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_help_num (bvw->priv->xine,
+			 "input.http_proxy_port", 8080, &entry);
+	entry.num_value = gconf_client_get_int (bvw->priv->gc,
+			"/system/http_proxy/port", NULL);
+	xine_config_update_entry (bvw->priv->xine, &entry);
 
-	memset (&entry, 0, sizeof (entry));
 	if (gconf_client_get_bool (bvw->priv->gc, "/system/http_proxy/use_authentication", NULL) == FALSE)
 	{
-		if (xine_config_lookup_entry (bvw->priv->xine,
-				"input.http_proxy_user", &entry))
-		{
-			entry.str_value = "";
-			xine_config_update_entry (bvw->priv->xine, &entry);
-		}
+		bvw_config_helper_string (bvw->priv->xine,
+				"input.http_proxy_user", g_get_user_name(),
+				&entry);
+		entry.str_value = "";
+		xine_config_update_entry (bvw->priv->xine, &entry);
 
-		memset (&entry, 0, sizeof (entry));
-		if (xine_config_lookup_entry (bvw->priv->xine,
-				"input.http_proxy_password", &entry))
-		{
-			entry.str_value = "";
-			xine_config_update_entry (bvw->priv->xine, &entry);
-		}
+		bvw_config_helper_string (bvw->priv->xine,
+				"input.http_proxy_password", "",
+				&entry);
+		entry.str_value = "";
+		xine_config_update_entry (bvw->priv->xine, &entry);
 	} else {
-		if (xine_config_lookup_entry (bvw->priv->xine,
-				"input.http_proxy_user", &entry))
-		{
-			entry.str_value = gconf_client_get_string
-				(bvw->priv->gc,
-				 "/system/http_proxy/authentication_user",
-				 NULL);
-			xine_config_update_entry (bvw->priv->xine, &entry);
-		}
+		bvw_config_helper_string (bvw->priv->xine,
+				"input.http_proxy_user", g_get_user_name(),
+				&entry);
+		entry.str_value = gconf_client_get_string (bvw->priv->gc,
+				"/system/http_proxy/authentication_user",
+				NULL);
+		xine_config_update_entry (bvw->priv->xine, &entry);
 
-		memset (&entry, 0, sizeof (entry));
-		if (xine_config_lookup_entry (bvw->priv->xine,
-				"input.http_proxy_password", &entry))
-		{
-			entry.str_value = gconf_client_get_string
-				(bvw->priv->gc,
-				 "/system/http_proxy/authentication_password",
-				 NULL);
-			xine_config_update_entry (bvw->priv->xine, &entry);
-		}
+		bvw_config_helper_string (bvw->priv->xine,
+				"input.http_proxy_password", "",
+				&entry);
+		entry.str_value = gconf_client_get_string (bvw->priv->gc,
+				"/system/http_proxy/authentication_password",
+				NULL);
+		xine_config_update_entry (bvw->priv->xine, &entry);
 	}
 }
 
@@ -741,12 +762,9 @@ setup_config_video (BaconVideoWidget *bvw)
 	xine_cfg_entry_t entry;
 
 	/* Default xv colourkey */
-	if (xine_config_lookup_entry (bvw->priv->xine,
-				"video.xv_colorkey", &entry))
-	{
-		entry.num_value = 30;
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_help_num (bvw->priv->xine, "video.xv_colorkey", 30, &entry);
+	entry.num_value = 30;
+	xine_config_update_entry (bvw->priv->xine, &entry);
 }
 
 static void
@@ -1079,6 +1097,11 @@ bacon_video_widget_idle_signal (BaconVideoWidget *bvw)
 				bvw_table_signals[ERROR],
 				0, data->msg, TRUE);
 		break;
+	case SPEED_WARNING_ASYNC:
+		g_signal_emit (G_OBJECT (bvw),
+				bvw_table_signals[SPEED_WARNING],
+				0, NULL);
+		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -1202,6 +1225,12 @@ xine_event (void *user_data, const xine_event_t *event)
 	case XINE_EVENT_UI_MESSAGE:
 		xine_event_message (bvw, (xine_ui_message_data_t *)event->data);
 		break;
+	case XINE_EVENT_DROPPED_FRAMES:
+		data = g_new0 (signal_data, 1);
+		data->signal = SPEED_WARNING_ASYNC;
+		g_async_queue_push (bvw->priv->queue, data);
+		g_idle_add ((GSourceFunc) bacon_video_widget_idle_signal, bvw);
+		break;
 	}
 }
 
@@ -1244,6 +1273,7 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 	BaconVideoWidget *bvw;
 	char *configfile;
 	xine_cfg_entry_t entry;
+	int speed;
 
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (widget));
@@ -1252,6 +1282,10 @@ bacon_video_widget_unrealize (GtkWidget *widget)
 
 	g_source_remove (bvw->priv->tick_id);
 
+	speed = xine_get_param (bvw->priv->stream, XINE_PARAM_SPEED);
+	if (speed != XINE_SPEED_PAUSE)
+		show_vfx_update (bvw, FALSE);
+
 	/* stop the playback */
 	xine_stop (bvw->priv->stream);
 	xine_close (bvw->priv->stream);
@@ -1462,9 +1496,10 @@ static gboolean
 bacon_video_widget_tick_send (BaconVideoWidget *bvw)
 {
 	int current_time, stream_length, current_position;
+	float current_position_f;
 	gboolean ret = TRUE;
 
-	if (bvw->priv->stream == NULL)
+	if (bvw->priv->stream == NULL || bvw->priv->logo_mode != FALSE)
 		return TRUE;
 
 	if (bvw->priv->mrl == NULL)
@@ -1479,10 +1514,28 @@ bacon_video_widget_tick_send (BaconVideoWidget *bvw)
 				&stream_length);
 	}
 
+	if (bvw->priv->seeking == 1)
+	{
+		current_position_f = bvw->priv->seek_dest;
+		current_time = bvw->priv->seek_dest * stream_length;
+	} else if (bvw->priv->seeking == 2) {
+		current_time = bvw->priv->seek_dest_time;
+		current_position_f = (float) current_time / stream_length;
+	} else {
+		current_position_f = (float) current_position / 65535;
+	}
+
+	if (stream_length > 0)
+		bvw->priv->is_live = FALSE;
+	else
+		bvw->priv->is_live = TRUE;
+
 	if (ret == TRUE)
 		g_signal_emit (G_OBJECT (bvw),
 				bvw_table_signals[TICK], 0,
-				current_time, stream_length, current_position);
+				(gint64) (current_time),
+				(gint64) (stream_length),
+				current_position_f);
 
 	return TRUE;
 }
@@ -1496,13 +1549,14 @@ show_vfx_update (BaconVideoWidget *bvw, gboolean show_visuals)
 	has_video = xine_get_stream_info(bvw->priv->stream,
 			XINE_STREAM_INFO_HAS_VIDEO);
 
-	if (has_video == TRUE && show_visuals == TRUE)
+	if (has_video != FALSE && show_visuals != FALSE
+			&& bvw->priv->using_vfx != FALSE)
 	{
 		audio_source = xine_get_audio_source (bvw->priv->stream);
 		if (xine_post_wire_audio_port (audio_source,
 					bvw->priv->ao_driver))
 			bvw->priv->using_vfx = FALSE;
-	} else if (has_video == FALSE && show_visuals == TRUE
+	} else if (has_video == FALSE && show_visuals != FALSE
 			&& bvw->priv->using_vfx == FALSE
 			&& bvw->priv->vis != NULL)
 	{
@@ -1645,21 +1699,41 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
 }
 
 gboolean
-bacon_video_widget_play (BaconVideoWidget *bvw, guint pos,
-		guint start_time, GError **gerror)
+bacon_video_widget_play (BaconVideoWidget *bvw, GError **gerror)
 {
-	int error, length;
+	int error;
 
 	g_return_val_if_fail (bvw != NULL, -1);
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), -1);
 	g_return_val_if_fail (bvw->priv->xine != NULL, -1);
 
 	bvw->priv->started = TRUE;
+	error = 1;
 
-	length = bacon_video_widget_get_stream_length (bvw);
+	if (bvw->priv->seeking == 1)
+	{
+		error = xine_play (bvw->priv->stream,
+				bvw->priv->seek_dest * 65535, 0);
+		bvw->priv->seeking = 0;
+	} else if (bvw->priv->seeking == 2) {
+		error = xine_play (bvw->priv->stream, 0,
+				bvw->priv->seek_dest_time);
+		bvw->priv->seeking = 0;
+	} else {
+		int speed, status;
 
-	error = xine_play (bvw->priv->stream, pos,
-			CLAMP (start_time, 0, length));
+		speed = xine_get_param (bvw->priv->stream, XINE_PARAM_SPEED);
+		status = xine_get_status (bvw->priv->stream);
+		if (speed != XINE_SPEED_NORMAL && status == XINE_STATUS_PLAY)
+		{
+			xine_set_param (bvw->priv->stream,
+					XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
+		} else {
+			error = xine_play (bvw->priv->stream, 0, 0);
+		}
+
+		bvw->priv->seeking = 0;
+	}
 
 	if (error == 0)
 	{
@@ -1677,6 +1751,68 @@ bacon_video_widget_play (BaconVideoWidget *bvw, guint pos,
 	return TRUE;
 }
 
+gboolean bacon_video_widget_seek (BaconVideoWidget *bvw, float position,
+		GError **gerror)
+{
+	int error, speed;
+
+	g_return_val_if_fail (bvw != NULL, -1);
+	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), -1);
+	g_return_val_if_fail (bvw->priv->xine != NULL, -1);
+
+	speed = xine_get_param (bvw->priv->stream, XINE_PARAM_SPEED);
+	if (speed == XINE_SPEED_PAUSE)
+	{
+		bvw->priv->seeking = 1;
+		bvw->priv->seek_dest = position;
+		return TRUE;
+	}
+
+	error = xine_play (bvw->priv->stream, position * 65535, 0);
+
+	if (error == 0)
+	{
+		xine_error (bvw, gerror);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+gboolean bacon_video_widget_seek_time (BaconVideoWidget *bvw, gint64 time,
+		GError **gerror)
+{
+	int error, speed;
+	gint64 length;
+
+	g_return_val_if_fail (bvw != NULL, -1);
+	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), -1);
+	g_return_val_if_fail (bvw->priv->xine != NULL, -1);
+
+	length = bacon_video_widget_get_stream_length (bvw);
+
+	speed = xine_get_param (bvw->priv->stream, XINE_PARAM_SPEED);
+	if (speed == XINE_SPEED_PAUSE)
+	{
+		bvw->priv->seeking = 2;
+		bvw->priv->seek_dest_time = CLAMP (time, 0, length);
+		return TRUE;
+	}
+
+	length = bacon_video_widget_get_stream_length (bvw);
+
+	error = xine_play (bvw->priv->stream, 0, CLAMP (time, 0, length));
+
+	if (error == 0)
+	{
+		xine_error (bvw, gerror);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+
 void
 bacon_video_widget_stop (BaconVideoWidget *bvw)
 {
@@ -1697,6 +1833,7 @@ bacon_video_widget_close (BaconVideoWidget *bvw)
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 	g_return_if_fail (bvw->priv->xine != NULL);
 
+	xine_stop (bvw->priv->stream);
 	xine_close (bvw->priv->stream);
 	g_free (bvw->priv->mrl);
 	bvw->priv->mrl = NULL;
@@ -1705,17 +1842,6 @@ bacon_video_widget_close (BaconVideoWidget *bvw)
 			bvw_table_signals[CHANNELS_CHANGE], 0, NULL);
 }
 
-gboolean
-bacon_video_widget_eject (BaconVideoWidget *bvw)
-{
-	g_return_val_if_fail (bvw != NULL, FALSE);
-	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
-	g_return_val_if_fail (bvw->priv->stream != NULL, FALSE);
-
-	bacon_video_widget_close (bvw);
-	return xine_eject (bvw->priv->stream);
-}
-
 /* Properties */
 static void
 bacon_video_widget_set_property (GObject *object, guint property_id,
@@ -1733,9 +1859,6 @@ bacon_video_widget_set_property (GObject *object, guint property_id,
 		bacon_video_widget_set_logo_mode (bvw,
 				g_value_get_boolean (value));
 		break;
-	case PROP_SPEED:
-		bacon_video_widget_set_speed (bvw, g_value_get_int (value));
-		break;
 	case PROP_SHOWCURSOR:
 		bacon_video_widget_set_show_cursor (bvw,
 				g_value_get_boolean (value));
@@ -1769,14 +1892,11 @@ bacon_video_widget_get_property (GObject *object, guint property_id,
 		g_value_set_boolean (value,
 				bacon_video_widget_get_logo_mode (bvw));
 		break;
-	case PROP_SPEED:
-		g_value_set_int (value, bacon_video_widget_get_speed (bvw));
-		break;
 	case PROP_POSITION:
-		g_value_set_int (value, bacon_video_widget_get_position (bvw));
+		g_value_set_int64 (value, bacon_video_widget_get_position (bvw));
 		break;
 	case PROP_STREAM_LENGTH:
-		g_value_set_int (value,
+		g_value_set_int64 (value,
 				bacon_video_widget_get_stream_length (bvw));
 		break;
 	case PROP_PLAYING:
@@ -1814,7 +1934,7 @@ bacon_video_widget_set_logo (BaconVideoWidget *bvw, char *filename)
 	g_return_if_fail (bvw->priv->xine != NULL);
 
 	if (bacon_video_widget_open (bvw, filename, NULL) == TRUE) {
-		bacon_video_widget_play (bvw, 0 , 0, NULL);
+		bacon_video_widget_play (bvw, NULL);
 	}
 }
 
@@ -1829,41 +1949,25 @@ bacon_video_widget_get_logo_mode (BaconVideoWidget *bvw)
 }
 
 void
-bacon_video_widget_set_speed (BaconVideoWidget *bvw, Speeds speed)
+bacon_video_widget_pause (BaconVideoWidget *bvw)
 {
 	g_return_if_fail (bvw != NULL);
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
 	g_return_if_fail (bvw->priv->xine != NULL);
 
-	xine_set_param (bvw->priv->stream, XINE_PARAM_SPEED, speeds[speed]);
+	xine_set_param (bvw->priv->stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
+
+	if (bvw->priv->is_live != FALSE)
+		xine_stop (bvw->priv->stream);
 
 #ifdef HAVE_XINE_CLOSE
 	/* Close the audio device when on pause */
-	if (speeds[speed] == XINE_SPEED_PAUSE)
-		xine_set_param (bvw->priv->stream,
-				XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
+	xine_set_param (bvw->priv->stream,
+			XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
 #endif
-
-	if (bvw->priv->queued_vis != NULL)
-	{
-		g_message ("removing %s from the queue (set_speed)", bvw->priv->queued_vis);
-		bacon_video_widget_set_visuals (bvw, bvw->priv->queued_vis);
-		g_free (bvw->priv->queued_vis);
-		bvw->priv->queued_vis = NULL;
-	}
 }
 
-int
-bacon_video_widget_get_speed (BaconVideoWidget *bvw)
-{
-	g_return_val_if_fail (bvw != NULL, SPEED_NORMAL);
-	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), SPEED_NORMAL);
-	g_return_val_if_fail (bvw->priv->xine != NULL, SPEED_NORMAL);
-
-	return xine_get_param (bvw->priv->stream, XINE_PARAM_SPEED);
-}
-
-int
+float
 bacon_video_widget_get_position (BaconVideoWidget *bvw)
 {
 	int pos_stream = 0, i = 0;
@@ -1891,19 +1995,17 @@ bacon_video_widget_get_position (BaconVideoWidget *bvw)
 		i++;
 	}
 
+	if (bvw->priv->seeking == 1)
+	{
+		return bvw->priv->seek_dest * length_time;
+	} else if (bvw->priv->seeking == 2) {
+		return bvw->priv->seek_dest_time;
+	}
+
 	if (ret == FALSE)
 		return -1;
 
-	return pos_stream;
-}
-
-gboolean
-bacon_video_widget_is_fullscreen (BaconVideoWidget *bvw)
-{
-	g_return_val_if_fail (bvw != NULL, 0);
-	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), 0);
-
-	return bvw->priv->fullscreen_mode;
+	return pos_stream / 65535;
 }
 
 gboolean
@@ -2014,30 +2116,22 @@ bacon_video_widget_set_media_device (BaconVideoWidget *bvw, const char *path)
 	xine_cfg_entry_t entry;
 
 	/* DVD device */
-	if (xine_config_lookup_entry (bvw->priv->xine,
-			"input.dvd_device", &entry))
-	{
-		entry.str_value = g_strdup (path);
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_helper_string (bvw->priv->xine, "input.dvd_device",
+			path, &entry);
+	entry.str_value = g_strdup (path);
+	xine_config_update_entry (bvw->priv->xine, &entry);
 
 	/* VCD device */
-	memset (&entry, 0, sizeof (entry));
-	if (xine_config_lookup_entry (bvw->priv->xine,
-				"input.vcd_device", &entry))
-	{
-		entry.str_value = g_strdup (path);
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_helper_string (bvw->priv->xine, "input.vcd_device",
+			path, &entry);
+	entry.str_value = g_strdup (path);
+	xine_config_update_entry (bvw->priv->xine, &entry);
 
 	/* CDDA device */
-	memset (&entry, 0, sizeof (entry));
-	if (xine_config_lookup_entry (bvw->priv->xine,
-				"input.cdda_device", &entry))
-	{
-		entry.str_value = g_strdup (path);
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_helper_string (bvw->priv->xine, "input.cdda_device",
+			path, &entry);
+	entry.str_value = g_strdup (path);
+	xine_config_update_entry (bvw->priv->xine, &entry);
 }
 
 void
@@ -2046,20 +2140,15 @@ bacon_video_widget_set_proprietary_plugins_path (BaconVideoWidget *bvw,
 {
 	xine_cfg_entry_t entry;
 
-	if (xine_config_lookup_entry (bvw->priv->xine,
-				"codec.win32_path", &entry))
-	{
-		entry.str_value = g_strdup (path);
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_helper_string (bvw->priv->xine,
+			"codec.win32_path", path, &entry);
+	entry.str_value = g_strdup (path);
+	xine_config_update_entry (bvw->priv->xine, &entry);
 
-	memset (&entry, 0, sizeof (entry));
-	if (xine_config_lookup_entry (bvw->priv->xine,
-				"codec.real_codecs_path", &entry))
-	{
-		entry.str_value = g_strdup (path);
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_helper_string (bvw->priv->xine,
+			"codec.real_codecs_path", path, &entry);
+	entry.str_value = g_strdup (path);
+	xine_config_update_entry (bvw->priv->xine, &entry);
 
 	/* And we try and create symlinks from /usr/lib/win32 to
 	 * the local user path */
@@ -2319,23 +2408,17 @@ bacon_video_widget_set_visuals_quality (BaconVideoWidget *bvw,
 		g_assert_not_reached ();
 	}
 
-	if (xine_config_lookup_entry (bvw->priv->xine, "post.goom_fps", &entry))
-	{
-		entry.num_value = fps;
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_help_num (bvw->priv->xine, "post.goom_fps", fps, &entry);
+	entry.num_value = fps;
+	xine_config_update_entry (bvw->priv->xine, &entry);
 
-	if (xine_config_lookup_entry (bvw->priv->xine, "post.goom_width", &entry))
-	{
-		entry.num_value = w;
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_help_num (bvw->priv->xine, "post.goom_width", w, &entry);
+	entry.num_value = w;
+	xine_config_update_entry (bvw->priv->xine, &entry);
 
-	if (xine_config_lookup_entry (bvw->priv->xine, "post.goom_height", &entry))
-	{
-		entry.num_value = h;
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_help_num (bvw->priv->xine, "post.goom_height", h, &entry);
+	entry.num_value = h;
+	xine_config_update_entry (bvw->priv->xine, &entry);
 }
 
 gboolean
@@ -2358,7 +2441,7 @@ bacon_video_widget_set_auto_resize (BaconVideoWidget *bvw, gboolean auto_resize)
 	bvw->priv->auto_resize = auto_resize;
 }
 
-int
+gint64
 bacon_video_widget_get_current_time (BaconVideoWidget *bvw)
 {
 	int pos_time = 0, i = 0;
@@ -2383,13 +2466,20 @@ bacon_video_widget_get_current_time (BaconVideoWidget *bvw)
 		i++;
 	}
 
+	if (bvw->priv->seeking == 1)
+	{
+		return bvw->priv->seek_dest * length_time;
+	} else if (bvw->priv->seeking == 2) {
+		return bvw->priv->seek_dest_time;
+	}
+
 	if (ret == FALSE)
 		return -1;
 
 	return pos_time;
 }
 
-int
+gint64
 bacon_video_widget_get_stream_length (BaconVideoWidget *bvw)
 {
 	int length_time = 0;
@@ -2418,7 +2508,7 @@ bacon_video_widget_is_playing (BaconVideoWidget *bvw)
 	if (bvw->priv->stream == NULL)
 		return FALSE;
 
-	return xine_get_status (bvw->priv->stream) == XINE_STATUS_PLAY;
+	return (xine_get_status (bvw->priv->stream) == XINE_STATUS_PLAY && xine_get_param (bvw->priv->stream, XINE_PARAM_SPEED) == XINE_SPEED_NORMAL);
 }
 
 gboolean
@@ -2651,30 +2741,21 @@ bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
 		g_assert_not_reached ();
 	}
 
-	if (xine_config_lookup_entry (bvw->priv->xine, "audio.four_channel", &entry))
-	{
-		entry.num_value = four_channel;
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_help_num (bvw->priv->xine, "audio.four_channel", four_channel, &entry);
+	entry.num_value = four_channel;
+	xine_config_update_entry (bvw->priv->xine, &entry);
 
-	if (xine_config_lookup_entry (bvw->priv->xine, "audio.five_channel", &entry))
-	{
-		entry.num_value = five_channel;
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_help_num (bvw->priv->xine, "audio.five_channel", five_channel,  &entry);
+	entry.num_value = five_channel;
+	xine_config_update_entry (bvw->priv->xine, &entry);
 
-	if (xine_config_lookup_entry (bvw->priv->xine, "audio.five_lfe_channel", &entry))
-	{
-		entry.num_value = five_one_channel;
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_help_num (bvw->priv->xine, "audio.five_lfe_channel", five_one_channel, &entry);
+	entry.num_value = five_one_channel;
+	xine_config_update_entry (bvw->priv->xine, &entry);
 
-	if (xine_config_lookup_entry (bvw->priv->xine, "audio.a52_pass_through",
- &entry))
-	{
-		entry.num_value = passthru;
-		xine_config_update_entry (bvw->priv->xine, &entry);
-	}
+	bvw_config_help_num (bvw->priv->xine, "audio.a52_pass_through", passthru, &entry);
+	entry.num_value = passthru;
+	xine_config_update_entry (bvw->priv->xine, &entry);
 }
 
 static void
