@@ -40,11 +40,97 @@
 #include <scsi/sg.h>
 #endif /* __linux__ */
 
+#ifdef __FreeBSD__
+#include <sys/cdio.h>
+#include <sys/cdrio.h>
+#include <camlib.h>
+#endif /* __FreeBSD__ */
+
 #include <glib.h>
 #include <libgnome/gnome-i18n.h>
 
 #include "cd-drive.h"
 
+static struct {
+	const char *name;
+	gboolean can_write_cdr;
+	gboolean can_write_cdrw;
+	gboolean can_write_dvdr;
+	gboolean can_write_dvdram;
+} recorder_whitelist[] = {
+	{ "IOMEGA - CDRW9602EXT-B", TRUE, TRUE, FALSE, FALSE },
+};
+
+#if defined(__linux__) || defined(__FreeBSD__)
+
+/* For dvd_plus_rw_utils.cpp */
+int get_dvd_r_rw_profile (const char *name);
+int get_mmc_profile (void *fd);
+
+static void
+add_dvd_plus (CDDrive *cdrom)
+{
+	int caps;
+
+	caps = get_dvd_r_rw_profile (cdrom->device);
+
+	if (caps == -1) {
+		return;
+	}
+
+	if (caps == 2) {
+		cdrom->type |= CDDRIVE_TYPE_DVD_PLUS_RW_RECORDER;
+		cdrom->type |= CDDRIVE_TYPE_DVD_PLUS_R_RECORDER;
+	} else if (caps == 0) {
+		cdrom->type |= CDDRIVE_TYPE_DVD_PLUS_R_RECORDER;
+	} else if (caps == 1) {
+		cdrom->type |= CDDRIVE_TYPE_DVD_PLUS_RW_RECORDER;
+	}
+}
+
+static CDMediaType
+linux_bsd_media_type (const char *device)
+{
+	int fd;
+	int mmc_profile;
+
+	fd = open (device, O_RDONLY);
+	if (fd < 0) {
+		return CD_MEDIA_TYPE_ERROR;
+	}
+
+	mmc_profile = get_mmc_profile ((void *)fd);
+	close (fd);
+
+	switch (mmc_profile) {
+	case -1:        /* Couldn't get the data about the media */
+		return CD_MEDIA_TYPE_UNKNOWN;
+	case 0x8:	/* Commercial CDs and Audio CD	*/
+		return CD_MEDIA_TYPE_CD;
+	case 0x9:	/* CD-R                         */
+		return CD_MEDIA_TYPE_CDR;
+	case 0xa:	/* CD-RW			*/
+		return CD_MEDIA_TYPE_CDRW;
+	case 0x10:	/* Commercial DVDs		*/
+		return CD_MEDIA_TYPE_DVD;
+	case 0x11:      /* DVD-R                        */
+		return CD_MEDIA_TYPE_DVDR;
+	case 0x13:      /* DVD-RW Restricted Overwrite  */
+	case 0x14:      /* DVD-RW Sequential            */
+		return CD_MEDIA_TYPE_DVDRW;
+	case 0x1B:      /* DVD+R                        */
+		return CD_MEDIA_TYPE_DVD_PLUS_R;
+	case 0x1A:      /* DVD+RW                       */
+		return CD_MEDIA_TYPE_DVD_PLUS_RW;
+	case 0x12:      /* DVD-RAM                      */
+		return CD_MEDIA_TYPE_DVD_RAM;
+	default:
+		return CD_MEDIA_TYPE_UNKNOWN;
+	}
+}
+
+#endif /* __linux__ || __FreeBSD__ */
+
 #ifdef __linux__
 
 #ifdef USE_STABLE_LIBGLIB
@@ -100,7 +186,6 @@ read_lines (char *filename)
 }
 
 struct scsi_unit {
-	gboolean exist;
 	char *vendor;
 	char *model;
 	char *rev;
@@ -112,6 +197,7 @@ struct scsi_unit {
 
 struct cdrom_unit {
 	char *device;
+	char *display_name;
 	int speed;
 	gboolean can_write_cdr;
 	gboolean can_write_cdrw;
@@ -120,39 +206,72 @@ struct cdrom_unit {
 	gboolean can_read_dvd;
 };
 
+static char *cdrom_get_name (struct cdrom_unit *cdrom, struct scsi_unit *scsi_units, int n_scsi_units);
+
 static void
-parse_sg_line (char *device_str, char *devices, struct scsi_unit *scsi_unit)
+add_whitelist (struct cdrom_unit *cdrom_s,
+	       struct scsi_unit *scsi_units, int n_scsi_units)
+{
+	guint i;
+
+	for (i = 0; i < G_N_ELEMENTS (recorder_whitelist); i++) {
+		if (cdrom_s->display_name == NULL) {
+			continue;
+		}
+
+		if (!strcmp (cdrom_s->display_name, recorder_whitelist[i].name)) {
+			cdrom_s->can_write_cdr =
+				recorder_whitelist[i].can_write_cdr;
+			cdrom_s->can_write_cdrw =
+				recorder_whitelist[i].can_write_cdrw;
+			cdrom_s->can_write_dvdr =
+				recorder_whitelist[i].can_write_dvdr;
+			cdrom_s->can_write_dvdram =
+				recorder_whitelist[i].can_write_dvdram;
+		}
+	}
+}
+
+static void
+get_scsi_units (char **device_str, char **devices, struct scsi_unit *scsi_units)
 {
 	char vendor[9], model[17], rev[5];
 	int host_no, access_count, queue_depth, device_busy, online, channel;
-	
-	scsi_unit->exist = FALSE;
-	
-	if (strcmp (device_str, "<no active device>") == 0) {
-		scsi_unit->exist = FALSE;
-		return;
-	}
-	if (sscanf (device_str, "%8c\t%16c\t%4c", vendor, model, rev) != 3) {
-		g_warning ("Couldn't match line in /proc/scsi/sg/device_strs\n");
-		return;
-	}
-	vendor[8] = 0; model[16] = 0; rev[4] = 0;
+	int scsi_id, scsi_lun, scsi_type;
+	int i, j;
 
-	scsi_unit->vendor = g_strdup (g_strstrip (vendor));
-	scsi_unit->model = g_strdup (g_strstrip (model));
-	scsi_unit->rev = g_strdup (g_strstrip (rev));
-	
-	if (sscanf (devices, "%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d",
-		    &host_no,
-		    &channel, &scsi_unit->id, &scsi_unit->lun, &scsi_unit->type,
-		    &access_count, &queue_depth, &device_busy,
-		    &online) != 9) {
+	for (i = 0, j = 0; device_str[i] != NULL && devices[i] != NULL; i++) {
+		if (strcmp (device_str[i], "<no active device>") == 0) {
+			continue;
+		}
+		if (sscanf (devices[i], "%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d",
+			    &host_no,
+			    &channel, &scsi_id, &scsi_lun, &scsi_type, &access_count, &queue_depth,
+				&device_busy, &online) != 9) {
 		
-		g_warning ("Couldn't match line in /proc/scsi/sg/devices\n");
-		return;
+			g_warning ("Couldn't match line in /proc/scsi/sg/devices\n");
+			continue;
+		}
+		if (scsi_type == 5) { /* TYPE_ROM (include/scsi/scsi.h) */
+			if (sscanf (device_str[i], "%8c\t%16c\t%4c", 
+						vendor, model, rev) != 3) {
+				g_warning ("Couldn't match line /proc/scsi/sg/device_strs\n");
+				continue;
+			}
+			vendor[8] = '\0'; model[16] = '\0'; rev[4] = '\0';
+
+			scsi_units[j].vendor = g_strdup (g_strstrip (vendor));
+			scsi_units[j].model = g_strdup (g_strstrip (model));
+			scsi_units[j].rev = g_strdup (g_strstrip (rev));
+			scsi_units[j].bus = host_no;
+			scsi_units[j].id = scsi_id;
+			scsi_units[j].lun = scsi_lun; 
+			scsi_units[j].type = scsi_type;
+			
+			j++;
+		}
 	}
-	scsi_unit->bus = host_no;
-	scsi_unit->exist = TRUE;
+	
 }
 
 static int
@@ -232,7 +351,7 @@ get_device_max_speed (char *id)
 	const char *argv[20]; /* Shouldn't need more than 20 arguments */
 	char *dev_str, *stdout_data, *speed;
 
-	max_speed = 1;
+	max_speed = -1;
 
 	i = 0;
 	argv[i++] = "cdrecord";
@@ -254,12 +373,18 @@ get_device_max_speed (char *id)
 		if (speed != NULL) {
 			speed += strlen ("Maximum write speed in kB/s:");
 			max_speed = (int)floor (atol (speed) / 176.0 + 0.5);
+		} else {
+			speed = strstr (stdout_data, "Maximum write speed:");
+			if (speed != NULL) {
+				speed += strlen ("Maximum write speed:");
+				max_speed = (int)floor (atol (speed) / 176.0 + 0.5);
+			}
 		}
+		g_free (stdout_data);
 	}
 
 	g_free (dev_str);
 	return max_speed;
-
 }
 
 
@@ -271,7 +396,7 @@ get_scsi_cd_name (int bus, int id, int lun, const char *dev,
 	
 	scsi_unit = lookup_scsi_unit (bus, id, lun, scsi_units, n_scsi_units);
 	if (scsi_unit == NULL) {
-		return g_strdup_printf (_("Unnamed SCSI CDROM (%s)"), dev);
+		return g_strdup_printf (_("Unnamed SCSI CD-ROM (%s)"), dev);
 	}
 
 	return g_strdup_printf ("%s - %s",
@@ -279,55 +404,32 @@ get_scsi_cd_name (int bus, int id, int lun, const char *dev,
 				scsi_unit->model);
 }
 
-static GList *
-add_linux_cd_recorder (GList *cdroms,
-		       struct cdrom_unit *cdrom_s,
-		       struct scsi_unit *scsi_units,
-		       int n_scsi_units)
-{
-	int bus, id, lun;
-	CDDrive *cdrom;
-
-	if (!get_cd_scsi_id (cdrom_s->device, &bus, &id, &lun)) {
-		return cdroms;
-	}
-
-	cdrom = g_new0 (CDDrive, 1);
-	cdrom->device = g_strdup_printf ("/dev/%s", cdrom_s->device);
-	cdrom->cdrecord_id = g_strdup_printf ("%d,%d,%d",
-					      bus, id, lun);
-	cdrom->display_name = get_scsi_cd_name (bus, id, lun, cdrom_s->device, scsi_units, n_scsi_units);
-	cdrom->max_speed_write = get_device_max_speed (cdrom->cdrecord_id);
-	cdrom->max_speed_read = cdrom_s->speed; 
-	if (cdrom_s->can_write_dvdr
-	    || cdrom_s->can_write_dvdram) {
-		cdrom->type = CDDRIVE_TYPE_DVD_RECORDER;
-	} else {
-		cdrom->type = CDDRIVE_TYPE_CD_RECORDER;
-	}
-	
-	return g_list_append (cdroms, cdrom);
-}
-
 static char *
 cdrom_get_name (struct cdrom_unit *cdrom, struct scsi_unit *scsi_units, int n_scsi_units)
 {
 	char *filename, *line, *retval;
+	char stdname[4], devfsname[15];
 	int bus, id, lun, i;
 
 	g_return_val_if_fail (cdrom != NULL, FALSE);
 
-	if ((cdrom->device[0] == 's' &&
-	    cdrom->device[1] == 'r') ||
-	    (cdrom->device[0] == 's' &&
-	    cdrom->device[1] == 'c' &&
-	    cdrom->device[2] == 'd')) {
+	/* clean up the string again if we have devfs */
+	i = sscanf(cdrom->device, "%4s %14s", stdname, devfsname);
+	if (i < 1) { /* should never happen */
+		g_warning("cdrom_get_name: cdrom->device string broken!");
+		return NULL;
+	}
+	if (i == 2) {
+		cdrom->device = g_strdup(devfsname);
+	}
+	stdname[3] = '\0'; devfsname[14] = '\0'; /* just in case */
+	
+	if (stdname[0] == 's') {
 		get_cd_scsi_id (cdrom->device, &bus, &id, &lun);
-
-		retval = get_scsi_cd_name (bus, id, lun, cdrom->device, scsi_units, n_scsi_units);
+		retval = get_scsi_cd_name (bus, id, lun, cdrom->device, scsi_units,
+			   n_scsi_units);
 	} else {
-		filename = g_strdup_printf ("/proc/ide/%s/model",
-					    cdrom->device);
+		filename = g_strdup_printf ("/proc/ide/%s/model", stdname);
 		if (!g_file_get_contents (filename, &line, NULL, NULL) ||
 		    line == NULL) {
 			g_free (filename);
@@ -348,6 +450,71 @@ cdrom_get_name (struct cdrom_unit *cdrom, struct scsi_unit *scsi_units, int n_sc
 	return retval;
 }
 
+static GList *
+add_linux_cd_recorder (GList *cdroms,
+		       gboolean recorder_only,
+		       struct cdrom_unit *cdrom_s,
+		       struct scsi_unit *scsi_units,
+		       int n_scsi_units)
+{
+	int bus, id, lun;
+	CDDrive *cdrom;
+
+	cdrom = g_new0 (CDDrive, 1);
+
+	cdrom->type = CDDRIVE_TYPE_CD_DRIVE;
+	cdrom->display_name = g_strdup (cdrom_s->display_name);
+
+	if (cdrom_s->device[0] == 's') {
+		if (!get_cd_scsi_id (cdrom_s->device, &bus, &id, &lun)) {
+			g_free (cdrom->display_name);
+			g_free (cdrom);
+			return cdroms;
+		}
+		cdrom->cdrecord_id = g_strdup_printf ("%d,%d,%d",
+				bus, id, lun);
+	} else {
+		/* kernel >=2.5 can write cd w/o ide-scsi */
+		cdrom->cdrecord_id = g_strdup_printf ("/dev/%s",
+				cdrom_s->device);
+	}
+
+	if (recorder_only) {
+		cdrom->max_speed_write = get_device_max_speed
+			(cdrom->cdrecord_id);
+		if (cdrom->max_speed_write == -1) {
+			cdrom->max_speed_write = cdrom_s->speed;
+		}
+	} else {
+		/* Have a wild guess, the drive should actually correct us */
+		cdrom->max_speed_write = cdrom_s->speed;
+	}
+
+	cdrom->device = g_strdup_printf ("/dev/%s", cdrom_s->device);
+	cdrom->max_speed_read = cdrom_s->speed;
+	if (cdrom_s->can_write_dvdr) {
+		cdrom->type |= CDDRIVE_TYPE_DVD_RW_RECORDER;
+	}
+
+	if (cdrom_s->can_write_dvdram) {
+		cdrom->type |= CDDRIVE_TYPE_DVD_RAM_RECORDER;
+	}
+
+	if (cdrom_s->can_write_cdr) {
+		cdrom->type |= CDDRIVE_TYPE_CD_RECORDER;
+	}
+	if (cdrom_s->can_write_cdrw) {
+		cdrom->type |= CDDRIVE_TYPE_CDRW_RECORDER;
+	}
+	if (cdrom_s->can_read_dvd) {
+		cdrom->type |= CDDRIVE_TYPE_DVD_DRIVE;
+	}
+
+	add_dvd_plus (cdrom);
+
+	return g_list_append (cdroms, cdrom);
+}
+
 static GList *
 add_linux_cd_drive (GList *cdroms, struct cdrom_unit *cdrom_s,
 		    struct scsi_unit *scsi_units, int n_scsi_units)
@@ -355,17 +522,16 @@ add_linux_cd_drive (GList *cdroms, struct cdrom_unit *cdrom_s,
 	CDDrive *cdrom;
 
 	cdrom = g_new0 (CDDrive, 1);
+	cdrom->type = CDDRIVE_TYPE_CD_DRIVE;
 	cdrom->cdrecord_id = NULL;
+	cdrom->display_name = g_strdup (cdrom_s->display_name);
 	cdrom->device = g_strdup_printf ("/dev/%s", cdrom_s->device);
-	cdrom->display_name = cdrom_get_name (cdrom_s, scsi_units, n_scsi_units);
 	cdrom->max_speed_write = 0; /* Can't write */
 	cdrom->max_speed_read = cdrom_s->speed;
 	if (cdrom_s->can_read_dvd) {
-		cdrom->type = CDDRIVE_TYPE_DVD_DRIVE;
-	} else {
-		cdrom->type = CDDRIVE_TYPE_CD_DRIVE;
+		cdrom->type |= CDDRIVE_TYPE_DVD_DRIVE;
 	}
-	
+
 	return g_list_append (cdroms, cdrom);
 }
 
@@ -374,7 +540,7 @@ get_cd_device_file (const char *str)
 {
 	char *devname;
 	
-	if (str[0] == 's' && str[1] == 'r') {
+	if (str[0] == 's') {
 		devname = g_strdup_printf ("/dev/scd%c", str[2]);
 		if (g_file_test (devname, G_FILE_TEST_EXISTS)) {
 			g_free (devname);
@@ -393,53 +559,29 @@ linux_scan (gboolean recorder_only)
 	struct scsi_unit *scsi_units;
 	struct cdrom_unit *cdroms;
 	char *p, *t;
-	int n_scsi_units, n_cdroms, i, j;
+	int n_cdroms, maj, min, i, j;
+	int n_scsi_units;
 	int fd;
+	FILE *file;
 	GList *cdroms_list;
+	gboolean have_devfs;
 
-	/* Open /dev/sg0 to force loading of the sg module if not loaded yet */
-	fd = open ("/dev/sg0", O_RDONLY);
-	if (fd != -1) {
-		close (fd);
+	/* devfs creates and populates the /dev/cdroms directory when its mounted
+	 * the 'old style names' are matched with devfs names below.
+	 * The cdroms.device string gets cleaned up again in cdrom_get_name()
+	 * we need the oldstyle name to get device->display_name for ide.
+	 */
+	have_devfs = FALSE;
+	if (g_file_test ("/dev/cdroms", G_FILE_TEST_IS_DIR)) {
+		have_devfs = TRUE;
 	}
-
-	devices = read_lines ("/proc/scsi/sg/devices");
-	if (devices != NULL) {
-		device_str = read_lines ("/proc/scsi/sg/device_strs");
-		if (device_str == NULL) {
-			g_warning ("Can't read /proc/scsi/sg/device_strs");
-			g_strfreev (devices);
-			return NULL;
-		}
-
-		/* Count the number of scsi units, DO NOT REMOVE */
-		for (n_scsi_units = 0;
-		     device_str[n_scsi_units] != NULL && devices[n_scsi_units] != NULL;
-		     n_scsi_units++) {
-			/* Nothing */
-		}
-
-		scsi_units = g_new0 (struct scsi_unit, n_scsi_units);
-		for (i = 0; i < n_scsi_units; i++) {
-			parse_sg_line (device_str[i], devices[i], &scsi_units[i]);
-		}
-
-		g_strfreev (device_str);
-		g_strfreev (devices);
-	} else {
-		scsi_units = NULL;
-		n_scsi_units = 0;
-	}
-
+	
 	cdrom_info = read_lines ("/proc/sys/dev/cdrom/info");
 	if (cdrom_info == NULL || cdrom_info[0] == NULL || cdrom_info[1] == NULL) {
 		g_warning ("Couldn't read /proc/sys/dev/cdrom/info");
-		g_free (scsi_units);
 		return NULL;
 	}
-
 	if (!g_str_has_prefix (cdrom_info[2], "drive name:\t")) {
-		g_free (scsi_units);
 		return NULL;
 	}
 	p = cdrom_info[2] + strlen ("drive name:\t");
@@ -460,6 +602,41 @@ linux_scan (gboolean recorder_only)
 		}
 	}
 
+	/* we only have to check the first char, since only ide or scsi 	
+	 * devices are listed in /proc/sys/dev/cdrom/info. It will always
+	 * be 'h' or 's'
+	 */
+	n_scsi_units = 0;
+	for (i = 0; i < n_cdroms; i++) {
+		if (cdroms[i].device[0] == 's') {
+			n_scsi_units++;
+		}
+	}
+
+	if (n_scsi_units > 0) {
+		/* open /dev/sg0 to force loading of the sg module if not loaded yet */
+		fd = open ("/dev/sg0", O_RDONLY);
+		if (fd != -1) {
+			close (fd);
+		}
+		
+		devices = read_lines ("/proc/scsi/sg/devices");
+		device_str = read_lines ("/proc/scsi/sg/device_strs");
+		if (device_str == NULL) {
+			g_warning ("Can't read /proc/scsi/sg/device_strs");
+			g_strfreev (devices);
+			return NULL;
+		}
+		
+		scsi_units = g_new0 (struct scsi_unit, n_scsi_units);
+		get_scsi_units (device_str, devices, scsi_units);
+
+		g_strfreev (device_str);
+		g_strfreev (devices);
+	} else {
+		scsi_units = NULL;
+	}
+	
 	for (i = 3; cdrom_info[i] != NULL; i++) {
 		if (g_str_has_prefix (cdrom_info[i], "Can write CD-R:")) {
 			p = cdrom_info[i] + strlen ("Can write CD-R:");
@@ -536,20 +713,43 @@ linux_scan (gboolean recorder_only)
 	}
 	g_strfreev (cdrom_info);
 
+	/* get kernel major.minor version */
+	file = fopen("/proc/sys/kernel/osrelease", "r");
+	if (file == NULL || fscanf(file, "%d.%d", &maj, &min) != 2) {
+		g_warning("Could not get kernel version.");
+		maj = min = 0;
+	}
+	fclose(file);
+
 	cdroms_list = NULL;
-	for (i = 0; i < n_cdroms; i++) {
-		if (cdroms[i].can_write_cdr ||
+	for (i = n_cdroms - 1, j = 0; i >= 0; i--, j++) {
+		if (have_devfs) {
+			cdroms[i].device = g_strdup_printf("%s cdroms/cdrom%d",
+					cdroms[i].device,  j);
+		}
+		cdroms[i].display_name = cdrom_get_name (&cdroms[i],
+				scsi_units, n_scsi_units);
+		add_whitelist (&cdroms[i], scsi_units, n_scsi_units);
+
+		if ((cdroms[i].can_write_cdr ||
 		    cdroms[i].can_write_cdrw ||
 		    cdroms[i].can_write_dvdr ||
-		    cdroms[i].can_write_dvdram) {
+		    cdroms[i].can_write_dvdram) &&
+			(cdroms[i].device[0] == 's' ||
+			(maj > 2) || (maj == 2 && min >= 5))) {
 			cdroms_list = add_linux_cd_recorder (cdroms_list,
-					&cdroms[i], scsi_units, n_scsi_units);
+					recorder_only, &cdroms[i],
+					scsi_units, n_scsi_units);
 		} else if (!recorder_only) {
 			cdroms_list = add_linux_cd_drive (cdroms_list,
 					&cdroms[i], scsi_units, n_scsi_units);
 		}
 	}
 
+	for (i = n_cdroms - 1; i >= 0; i--) {
+		g_free (cdroms[i].display_name);
+	}
+
 	g_free (scsi_units);
 	g_free (cdroms);
 
@@ -557,6 +757,87 @@ linux_scan (gboolean recorder_only)
 }
 #endif /* __linux__ */
 
+#ifdef __FreeBSD__
+static GList *
+freebsd_scan (gboolean recorder_only)
+{
+	GList *cdroms_list = NULL;
+	const char *dev_type = "cd";
+	int fd;
+	int speed = 16; /* XXX Hardcode the write speed for now. */
+	int max_speed = CDR_MAX_SPEED;
+	int i = 0;
+
+	while (1) {
+		CDDriveType type;
+		CDDrive *cdrom;
+		gchar *cam_path, *dev;
+		struct cam_device *cam_dev;
+
+		type = 0;
+		cam_path = g_strdup_printf ("/dev/%s%dc", dev_type, i);
+
+		if (!g_file_test (cam_path, G_FILE_TEST_EXISTS)) {
+			g_free (cam_path);
+			break;
+		}
+
+		if ((cam_dev = cam_open_spec_device (dev_type, i, O_RDWR, NULL)) == NULL) {
+			i++;
+			g_free (cam_path);
+			continue;
+		}
+
+		/* XXX Other controllers might need to be added. */
+		if ((strncmp (cam_dev->sim_name, "ata", 3)) == 0) {
+			dev = g_strdup_printf ("/dev/a%s%dc", dev_type, i);
+		} else {
+			dev = g_strdup (cam_path);
+		}
+
+		g_free (cam_path);
+
+		if ((fd = open (dev, O_RDWR, 0)) < 0) {
+			g_free (dev);
+			free (cam_dev);
+			i++;
+			continue;
+		}
+
+		type |= CDDRIVE_TYPE_CD_DRIVE;
+		if (ioctl (fd, CDRIOCWRITESPEED, &max_speed) >= 0) {
+			/* XXX is it CD-R or CD-RW ? */
+			type |= CDDRIVE_TYPE_CD_RECORDER;
+		}
+
+		close (fd);
+
+		if (type & CDDRIVE_TYPE_CD_RECORDER
+				|| type & CDDRIVE_TYPE_CDRW_RECORDER
+				|| recorder_only == FALSE) {
+			cdrom = g_new0 (CDDrive, 1);
+			cdrom->display_name = g_strdup_printf ("%s %s", cam_dev->inq_data.vendor, cam_dev->inq_data.revision);
+			cdrom->device = g_strdup (dev);
+			cdrom->max_speed_read = speed;
+			cdrom->max_speed_write = speed;
+			cdrom->cdrecord_id = g_strdup_printf ("%d,%d,%d", cam_dev->path_id, cam_dev->target_id, cam_dev->target_lun);
+			cdrom->type = type;
+
+			add_dvd_plus (cdrom);
+
+			cdroms_list = g_list_append (cdroms_list, cdrom);
+		}
+
+		g_free (dev);
+		free (cam_dev);
+
+		i++;
+	}
+
+	return cdroms_list;
+}
+#endif /* __FreeBSD__ */
+
 GList *
 scan_for_cdroms (gboolean recorder_only, gboolean add_image)
 {
@@ -567,6 +848,10 @@ scan_for_cdroms (gboolean recorder_only, gboolean add_image)
 	cdroms = linux_scan (recorder_only);
 #endif
 
+#ifdef __FreeBSD__
+	cdroms = freebsd_scan (recorder_only);
+#endif
+
 	if (add_image) {
 		/* File */
 		cdrom = g_new0 (CDDrive, 1);
@@ -581,3 +866,23 @@ scan_for_cdroms (gboolean recorder_only, gboolean add_image)
 	return cdroms;
 }
 
+void
+cd_drive_free (CDDrive *drive)
+{
+	g_return_if_fail (drive != NULL);
+
+	g_free (drive->display_name);
+	g_free (drive->cdrecord_id);
+	g_free (drive->device);
+}
+
+CDMediaType
+guess_media_type (const char *device_path)
+{
+#if defined(__linux__) || defined(__FreeBSD__)
+	return linux_bsd_media_type (device_path);
+#else
+	return CD_BLANK_MEDIA_TYPE_UNKNOWN;
+#endif
+}
+
